import typing

import httpx

from .http_client import AsyncHttpClient, HttpClient


class BaseClientWrapper:
    def __init__(
        self,
        *,
        api_key: str,
        base_url: str,
        timeout: typing.Optional[float] = None,
    ):
        self._base_url = base_url
        self._timeout = timeout

        # even in the async case, refreshing access token (when the existing one is expired) should be sync
        from ..tokens.client_ext import TokensClientExt
        self._tokens_client = TokensClientExt(base_url=base_url, api_key=api_key)


    def get_timeout(self) -> typing.Optional[float]:
        return self._timeout


    def get_base_url(self) -> str:
        return self._base_url


    def get_headers(self) -> typing.Dict[str, str]:
        headers: typing.Dict[str, str] = {
            "X-Fern-Language": "Python",
            "X-Fern-SDK-Name": "label-studio-sdk",
            "X-Fern-SDK-Version": "1.0.11",
        }
        if self._tokens_client._use_legacy_token:
            headers["Authorization"] = f"Token {self._tokens_client.api_key}"
        else:
            headers["Authorization"] = f"Bearer {self._tokens_client.api_key}"
        return headers


class SyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        api_key: str,
        base_url: str,
        timeout: typing.Optional[float] = None,
        httpx_client: httpx.Client,
    ):
        super().__init__(api_key=api_key, base_url=base_url, timeout=timeout)
        self.httpx_client = HttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
        )


class AsyncClientWrapper(BaseClientWrapper):
    def __init__(
        self, *, api_key: str, base_url: str, timeout: typing.Optional[float] = None, httpx_client: httpx.AsyncClient
    ):
        super().__init__(api_key=api_key, base_url=base_url, timeout=timeout)
        self.httpx_client = AsyncHttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
        )



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.s3s_export_storage import S3SExportStorage
from ...core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.jsonable_encoder import jsonable_encoder
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class S3SClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, project: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[S3SExportStorage]:
        """

        You can connect your S3 bucket to Label Studio as a source storage or target storage. Use this API request to get a list of all S3 export (target) storage connections for a specific project.

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](../projects/list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[S3SExportStorage]


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.s3s.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/s3s",
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[S3SExportStorage],
                    parse_obj_as(
                        type_=typing.List[S3SExportStorage],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        bucket: typing.Optional[str] = OMIT,
        prefix: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        role_arn: typing.Optional[str] = OMIT,
        region_name: typing.Optional[str] = OMIT,
        s3endpoint: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> S3SExportStorage:
        """

        Create a new target storage connection to a S3 bucket with IAM role access.

        For information about the required fields and prerequisites, see [Amazon S3](https://docs.humansignal.com/guide/storage#Set-up-an-S3-connection-with-IAM-role-access) in the Label Studio documentation.

        Parameters
        ----------
        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        bucket : typing.Optional[str]
            S3 bucket name

        prefix : typing.Optional[str]
            S3 bucket prefix

        external_id : typing.Optional[str]
            AWS External ID

        role_arn : typing.Optional[str]
            AWS Role ARN

        region_name : typing.Optional[str]
            AWS Region

        s3endpoint : typing.Optional[str]
            S3 Endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        S3SExportStorage


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.s3s.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/s3s",
            method="POST",
            json={
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "bucket": bucket,
                "prefix": prefix,
                "external_id": external_id,
                "role_arn": role_arn,
                "region_name": region_name,
                "s3_endpoint": s3endpoint,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    S3SExportStorage,
                    parse_obj_as(
                        type_=S3SExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> S3SExportStorage:
        """

        Get a specific S3 export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Parameters
        ----------
        id : int
            Export storage ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        S3SExportStorage


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.s3s.get(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/s3s/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    S3SExportStorage,
                    parse_obj_as(
                        type_=S3SExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete a specific S3 export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Parameters
        ----------
        id : int
            Export storage ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.s3s.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/s3s/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: int,
        *,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        bucket: typing.Optional[str] = OMIT,
        prefix: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        role_arn: typing.Optional[str] = OMIT,
        region_name: typing.Optional[str] = OMIT,
        s3endpoint: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> S3SExportStorage:
        """

        Update a specific S3 export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Parameters
        ----------
        id : int
            Export storage ID

        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        bucket : typing.Optional[str]
            S3 bucket name

        prefix : typing.Optional[str]
            S3 bucket prefix

        external_id : typing.Optional[str]
            AWS External ID

        role_arn : typing.Optional[str]
            AWS Role ARN

        region_name : typing.Optional[str]
            AWS Region

        s3endpoint : typing.Optional[str]
            S3 Endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        S3SExportStorage


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.s3s.update(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/storages/export/s3s/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "bucket": bucket,
                "prefix": prefix,
                "external_id": external_id,
                "role_arn": role_arn,
                "region_name": region_name,
                "s3_endpoint": s3endpoint,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    S3SExportStorage,
                    parse_obj_as(
                        type_=S3SExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def validate(
        self,
        *,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        bucket: typing.Optional[str] = OMIT,
        prefix: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        role_arn: typing.Optional[str] = OMIT,
        region_name: typing.Optional[str] = OMIT,
        s3endpoint: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """

        Validate a specific S3 export storage connection. This is useful to ensure that the storage configuration settings are correct and operational before attempting to export data.

        Parameters
        ----------
        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        bucket : typing.Optional[str]
            S3 bucket name

        prefix : typing.Optional[str]
            S3 bucket prefix

        external_id : typing.Optional[str]
            AWS External ID

        role_arn : typing.Optional[str]
            AWS Role ARN

        region_name : typing.Optional[str]
            AWS Region

        s3endpoint : typing.Optional[str]
            S3 Endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.export_storage.s3s.validate()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/storages/export/s3s/validate",
            method="POST",
            json={
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "bucket": bucket,
                "prefix": prefix,
                "external_id": external_id,
                "role_arn": role_arn,
                "region_name": region_name,
                "s3_endpoint": s3endpoint,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncS3SClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, project: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[S3SExportStorage]:
        """

        You can connect your S3 bucket to Label Studio as a source storage or target storage. Use this API request to get a list of all S3 export (target) storage connections for a specific project.

        The project ID can be found in the URL when viewing the project in Label Studio, or you can retrieve all project IDs using [List all projects](../projects/list).

        For more information about working with external storage, see [Sync data from external storage](https://labelstud.io/guide/storage).

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[S3SExportStorage]


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.s3s.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/s3s",
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[S3SExportStorage],
                    parse_obj_as(
                        type_=typing.List[S3SExportStorage],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        bucket: typing.Optional[str] = OMIT,
        prefix: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        role_arn: typing.Optional[str] = OMIT,
        region_name: typing.Optional[str] = OMIT,
        s3endpoint: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> S3SExportStorage:
        """

        Create a new target storage connection to a S3 bucket with IAM role access.

        For information about the required fields and prerequisites, see [Amazon S3](https://docs.humansignal.com/guide/storage#Set-up-an-S3-connection-with-IAM-role-access) in the Label Studio documentation.

        Parameters
        ----------
        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        bucket : typing.Optional[str]
            S3 bucket name

        prefix : typing.Optional[str]
            S3 bucket prefix

        external_id : typing.Optional[str]
            AWS External ID

        role_arn : typing.Optional[str]
            AWS Role ARN

        region_name : typing.Optional[str]
            AWS Region

        s3endpoint : typing.Optional[str]
            S3 Endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        S3SExportStorage


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.s3s.create()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/s3s",
            method="POST",
            json={
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "bucket": bucket,
                "prefix": prefix,
                "external_id": external_id,
                "role_arn": role_arn,
                "region_name": region_name,
                "s3_endpoint": s3endpoint,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    S3SExportStorage,
                    parse_obj_as(
                        type_=S3SExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> S3SExportStorage:
        """

        Get a specific S3 export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Parameters
        ----------
        id : int
            Export storage ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        S3SExportStorage


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.s3s.get(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/s3s/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    S3SExportStorage,
                    parse_obj_as(
                        type_=S3SExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete a specific S3 export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Parameters
        ----------
        id : int
            Export storage ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.s3s.delete(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/s3s/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: int,
        *,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        bucket: typing.Optional[str] = OMIT,
        prefix: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        role_arn: typing.Optional[str] = OMIT,
        region_name: typing.Optional[str] = OMIT,
        s3endpoint: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> S3SExportStorage:
        """

        Update a specific S3 export storage connection. You will need to provide the export storage ID. You can find this using [List export storages](list).

        Parameters
        ----------
        id : int
            Export storage ID

        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        bucket : typing.Optional[str]
            S3 bucket name

        prefix : typing.Optional[str]
            S3 bucket prefix

        external_id : typing.Optional[str]
            AWS External ID

        role_arn : typing.Optional[str]
            AWS Role ARN

        region_name : typing.Optional[str]
            AWS Region

        s3endpoint : typing.Optional[str]
            S3 Endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        S3SExportStorage


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.s3s.update(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/storages/export/s3s/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "bucket": bucket,
                "prefix": prefix,
                "external_id": external_id,
                "role_arn": role_arn,
                "region_name": region_name,
                "s3_endpoint": s3endpoint,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    S3SExportStorage,
                    parse_obj_as(
                        type_=S3SExportStorage,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def validate(
        self,
        *,
        can_delete_objects: typing.Optional[bool] = OMIT,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        project: typing.Optional[int] = OMIT,
        bucket: typing.Optional[str] = OMIT,
        prefix: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        role_arn: typing.Optional[str] = OMIT,
        region_name: typing.Optional[str] = OMIT,
        s3endpoint: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """

        Validate a specific S3 export storage connection. This is useful to ensure that the storage configuration settings are correct and operational before attempting to export data.

        Parameters
        ----------
        can_delete_objects : typing.Optional[bool]
            Deletion from storage enabled.

        title : typing.Optional[str]
            Storage title

        description : typing.Optional[str]
            Storage description

        project : typing.Optional[int]
            Project ID

        bucket : typing.Optional[str]
            S3 bucket name

        prefix : typing.Optional[str]
            S3 bucket prefix

        external_id : typing.Optional[str]
            AWS External ID

        role_arn : typing.Optional[str]
            AWS Role ARN

        region_name : typing.Optional[str]
            AWS Region

        s3endpoint : typing.Optional[str]
            S3 Endpoint

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.export_storage.s3s.validate()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/storages/export/s3s/validate",
            method="POST",
            json={
                "can_delete_objects": can_delete_objects,
                "title": title,
                "description": description,
                "project": project,
                "bucket": bucket,
                "prefix": prefix,
                "external_id": external_id,
                "role_arn": role_arn,
                "region_name": region_name,
                "s3_endpoint": s3endpoint,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .prompts_batch_failed_predictions_request_failed_predictions_item import (
    PromptsBatchFailedPredictionsRequestFailedPredictionsItem,
)
from .prompts_batch_failed_predictions_response import PromptsBatchFailedPredictionsResponse
from .prompts_batch_predictions_request_results_item import PromptsBatchPredictionsRequestResultsItem
from .prompts_batch_predictions_response import PromptsBatchPredictionsResponse

__all__ = [
    "PromptsBatchFailedPredictionsRequestFailedPredictionsItem",
    "PromptsBatchFailedPredictionsResponse",
    "PromptsBatchPredictionsRequestResultsItem",
    "PromptsBatchPredictionsResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .key_indicators_item_additional_kpis_item import KeyIndicatorsItemAdditionalKpisItem
from .key_indicators_item_extra_kpis_item import KeyIndicatorsItemExtraKpisItem
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class KeyIndicatorsItem(UniversalBaseModel):
    key: str = pydantic.Field()
    """
    The key for this KPI, where you can find the value from inside main_kpi
    """

    title: str = pydantic.Field()
    """
    The title for this metric, to be displayed to the user
    """

    main_kpi: str
    secondary_kpi: typing.Optional[str] = None
    additional_kpis: typing.Optional[typing.List[KeyIndicatorsItemAdditionalKpisItem]] = pydantic.Field(default=None)
    """
    Additional KPIs to be displayed at the bottom of the box
    """

    extra_kpis: typing.Optional[typing.List[KeyIndicatorsItemExtraKpisItem]] = pydantic.Field(default=None)
    """
    Extra KPIs to be displayed in the hover-tootip for that indicator
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import datetime as dt
from .annotations_dm_field_last_action import AnnotationsDmFieldLastAction
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class AnnotationsDmField(UniversalBaseModel):
    id: typing.Optional[int] = None
    result: typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]] = pydantic.Field(default=None)
    """
    List of annotation results for the task
    """

    created_username: typing.Optional[str] = pydantic.Field(default=None)
    """
    Username string
    """

    created_ago: typing.Optional[str] = pydantic.Field(default=None)
    """
    Time delta from creation time
    """

    completed_by: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    User details who completed this annotation.
    """

    unique_id: typing.Optional[str] = None
    was_cancelled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    User skipped the task
    """

    ground_truth: typing.Optional[bool] = pydantic.Field(default=None)
    """
    This annotation is a Ground Truth (ground_truth)
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation time
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last updated time
    """

    draft_created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Draft creation time
    """

    lead_time: typing.Optional[float] = pydantic.Field(default=None)
    """
    How much time it took to annotate the task
    """

    import_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Original annotation ID that was at the import step or NULL if this annotation wasn't imported
    """

    last_action: typing.Optional[AnnotationsDmFieldLastAction] = pydantic.Field(default=None)
    """
    Action which was performed in the last annotation history item
    """

    task: typing.Optional[int] = pydantic.Field(default=None)
    """
    Corresponding task for this annotation
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID for this annotation
    """

    updated_by: typing.Optional[int] = pydantic.Field(default=None)
    """
    Last user who updated this annotation
    """

    parent_prediction: typing.Optional[int] = pydantic.Field(default=None)
    """
    Points to the prediction from which this annotation was created
    """

    parent_annotation: typing.Optional[int] = pydantic.Field(default=None)
    """
    Points to the parent annotation from which this annotation was created
    """

    last_created_by: typing.Optional[int] = pydantic.Field(default=None)
    """
    User who created the last annotation history item
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class DataManagerTaskSerializerPredictionsItem(UniversalBaseModel):
    result: typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]] = None
    score: typing.Optional[float] = None
    model_version: typing.Optional[str] = None
    model: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    model_run: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    task: typing.Optional[int] = None
    project: typing.Optional[float] = None
    created_at: typing.Optional[dt.datetime] = None
    updated_at: typing.Optional[dt.datetime] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

PromptVersionProvider = typing.Union[
    typing.Literal["OpenAI", "AzureOpenAI", "AzureAIFoundry", "VertexAI", "Gemini", "Anthropic", "Custom"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .members.client import MembersClient
from ..core.request_options import RequestOptions
from ..types.workspace import Workspace
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.jsonable_encoder import jsonable_encoder
from ..core.client_wrapper import AsyncClientWrapper
from .members.client import AsyncMembersClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WorkspacesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.members = MembersClient(client_wrapper=self._client_wrapper)

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[Workspace]:
        """

        List all workspaces for your organization.

        Workspaces in Label Studio let you organize your projects and users into separate spaces. This is useful for managing different teams, departments, or projects within your organization.

        For more information, see [Workspaces in Label Studio](https://docs.humansignal.com/guide/workspaces).

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Workspace]


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.workspaces.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/workspaces",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Workspace],
                    parse_obj_as(
                        type_=typing.List[Workspace],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        is_personal: typing.Optional[bool] = OMIT,
        color: typing.Optional[str] = OMIT,
        is_archived: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Workspace:
        """

        Create a new workspace.

        Workspaces in Label Studio let you organize your projects and users into separate spaces. This is useful for managing different teams, departments, or projects within your organization.

        For more information, see [Workspaces in Label Studio](https://docs.humansignal.com/guide/workspaces).

        Parameters
        ----------
        title : typing.Optional[str]
            Workspace title

        description : typing.Optional[str]
            Workspace description

        is_public : typing.Optional[bool]
            Is workspace public

        is_personal : typing.Optional[bool]
            Is workspace personal

        color : typing.Optional[str]
            Workspace color in HEX format

        is_archived : typing.Optional[bool]
            Is workspace archived

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Workspace


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.workspaces.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/workspaces",
            method="POST",
            json={
                "title": title,
                "description": description,
                "is_public": is_public,
                "is_personal": is_personal,
                "color": color,
                "is_archived": is_archived,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Workspace,
                    parse_obj_as(
                        type_=Workspace,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Workspace:
        """

        Get information about a specific workspace. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Workspace


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.workspaces.get(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Workspace,
                    parse_obj_as(
                        type_=Workspace,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete a specific workspace. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.workspaces.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: int,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        is_personal: typing.Optional[bool] = OMIT,
        color: typing.Optional[str] = OMIT,
        is_archived: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Workspace:
        """

        Update a specific workspace. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        title : typing.Optional[str]
            Workspace title

        description : typing.Optional[str]
            Workspace description

        is_public : typing.Optional[bool]
            Is workspace public

        is_personal : typing.Optional[bool]
            Is workspace personal

        color : typing.Optional[str]
            Workspace color in HEX format

        is_archived : typing.Optional[bool]
            Is workspace archived

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Workspace


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.workspaces.update(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "is_public": is_public,
                "is_personal": is_personal,
                "color": color,
                "is_archived": is_archived,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Workspace,
                    parse_obj_as(
                        type_=Workspace,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncWorkspacesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.members = AsyncMembersClient(client_wrapper=self._client_wrapper)

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[Workspace]:
        """

        List all workspaces for your organization.

        Workspaces in Label Studio let you organize your projects and users into separate spaces. This is useful for managing different teams, departments, or projects within your organization.

        For more information, see [Workspaces in Label Studio](https://docs.humansignal.com/guide/workspaces).

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Workspace]


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspaces.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/workspaces",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Workspace],
                    parse_obj_as(
                        type_=typing.List[Workspace],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        is_personal: typing.Optional[bool] = OMIT,
        color: typing.Optional[str] = OMIT,
        is_archived: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Workspace:
        """

        Create a new workspace.

        Workspaces in Label Studio let you organize your projects and users into separate spaces. This is useful for managing different teams, departments, or projects within your organization.

        For more information, see [Workspaces in Label Studio](https://docs.humansignal.com/guide/workspaces).

        Parameters
        ----------
        title : typing.Optional[str]
            Workspace title

        description : typing.Optional[str]
            Workspace description

        is_public : typing.Optional[bool]
            Is workspace public

        is_personal : typing.Optional[bool]
            Is workspace personal

        color : typing.Optional[str]
            Workspace color in HEX format

        is_archived : typing.Optional[bool]
            Is workspace archived

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Workspace


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspaces.create()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/workspaces",
            method="POST",
            json={
                "title": title,
                "description": description,
                "is_public": is_public,
                "is_personal": is_personal,
                "color": color,
                "is_archived": is_archived,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Workspace,
                    parse_obj_as(
                        type_=Workspace,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Workspace:
        """

        Get information about a specific workspace. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Workspace


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspaces.get(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Workspace,
                    parse_obj_as(
                        type_=Workspace,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete a specific workspace. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspaces.delete(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: int,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        is_public: typing.Optional[bool] = OMIT,
        is_personal: typing.Optional[bool] = OMIT,
        color: typing.Optional[str] = OMIT,
        is_archived: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Workspace:
        """

        Update a specific workspace. You will need to provide the workspace ID. You can find this using [List workspaces](list).

        Parameters
        ----------
        id : int
            Workspace ID

        title : typing.Optional[str]
            Workspace title

        description : typing.Optional[str]
            Workspace description

        is_public : typing.Optional[bool]
            Is workspace public

        is_personal : typing.Optional[bool]
            Is workspace personal

        color : typing.Optional[str]
            Workspace color in HEX format

        is_archived : typing.Optional[bool]
            Is workspace archived

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Workspace


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspaces.update(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/workspaces/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "is_public": is_public,
                "is_personal": is_personal,
                "color": color,
                "is_archived": is_archived,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Workspace,
                    parse_obj_as(
                        type_=Workspace,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

ExportFormat = typing.Union[
    typing.Literal[
        "JSON",
        "JSON_MIN",
        "CSV",
        "TSV",
        "CONLL2003",
        "COCO",
        "VOC",
        "BRUSH_TO_NUMPY",
        "BRUSH_TO_PNG",
        "ASR_MANIFEST",
        "YOLO",
        "YOLO_OBB",
        "CSV_OLD",
        "YOLO_WITH_IMAGES",
        "COCO_WITH_IMAGES",
        "YOLO_OBB_WITH_IMAGES",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class GcsUpdateResponse(UniversalBaseModel):
    can_delete_objects: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Deletion from storage enabled.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage description
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID
    """

    bucket: typing.Optional[str] = pydantic.Field(default=None)
    """
    GCS bucket name
    """

    prefix: typing.Optional[str] = pydantic.Field(default=None)
    """
    GCS bucket prefix
    """

    google_application_credentials: typing.Optional[str] = pydantic.Field(default=None)
    """
    The content of GOOGLE_APPLICATION_CREDENTIALS json file. Check official Google Cloud Authentication documentation for more details.
    """

    google_project_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Google project ID
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .model_provider_connection_provider import ModelProviderConnectionProvider
import typing
from .model_provider_connection_scope import ModelProviderConnectionScope
from .model_provider_connection_organization import ModelProviderConnectionOrganization
from .model_provider_connection_created_by import ModelProviderConnectionCreatedBy
import datetime as dt
import pydantic
from .model_provider_connection_budget_reset_period import ModelProviderConnectionBudgetResetPeriod
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ModelProviderConnection(UniversalBaseModel):
    provider: ModelProviderConnectionProvider
    api_key: typing.Optional[str] = None
    deployment_name: typing.Optional[str] = None
    endpoint: typing.Optional[str] = None
    scope: typing.Optional[ModelProviderConnectionScope] = None
    organization: typing.Optional[ModelProviderConnectionOrganization] = None
    created_by: typing.Optional[ModelProviderConnectionCreatedBy] = None
    created_at: typing.Optional[dt.datetime] = None
    updated_at: typing.Optional[dt.datetime] = None
    is_internal: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the model provider connection is internal, not visible to the user.
    """

    budget_limit: typing.Optional[float] = pydantic.Field(default=None)
    """
    Budget limit for the model provider connection (null if unlimited)
    """

    budget_last_reset_date: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Date and time the budget was last reset
    """

    budget_reset_period: typing.Optional[ModelProviderConnectionBudgetResetPeriod] = pydantic.Field(default=None)
    """
    Budget reset period for the model provider connection (null if not reset)
    """

    budget_total_spent: typing.Optional[float] = pydantic.Field(default=None)
    """
    Tracked total budget spent for the given provider connection within the current budget period
    """

    budget_alert_threshold: typing.Optional[float] = pydantic.Field(default=None)
    """
    Budget alert threshold for the given provider connection
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Filter(UniversalBaseModel):
    id: typing.Optional[int] = None
    index: typing.Optional[int] = pydantic.Field(default=None)
    """
    To keep filter order
    """

    column: str = pydantic.Field()
    """
    Field name
    """

    type: str = pydantic.Field()
    """
    Field type
    """

    operator: str = pydantic.Field()
    """
    Filter operator
    """

    value: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Filter value
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .ml_backend_state import MlBackendState
import pydantic
from .ml_backend_auth_method import MlBackendAuthMethod
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class MlBackend(UniversalBaseModel):
    id: typing.Optional[int] = None
    state: typing.Optional[MlBackendState] = None
    readable_state: typing.Optional[str] = None
    is_interactive: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Used to interactively annotate tasks. If true, model returns one list with results
    """

    url: str = pydantic.Field()
    """
    URL for the machine learning model server
    """

    error_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message in error state
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the machine learning backend
    """

    auth_method: typing.Optional[MlBackendAuthMethod] = None
    basic_auth_user: typing.Optional[str] = pydantic.Field(default=None)
    """
    HTTP Basic Auth user
    """

    basic_auth_pass: typing.Optional[str] = None
    basic_auth_pass_is_set: typing.Optional[str] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description for the machine learning backend
    """

    extra_params: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Any extra parameters passed to the ML Backend during the setup
    """

    model_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    Current model version associated with this machine learning backend
    """

    timeout: typing.Optional[float] = pydantic.Field(default=None)
    """
    Response model timeout
    """

    created_at: typing.Optional[dt.datetime] = None
    updated_at: typing.Optional[dt.datetime] = None
    auto_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If false, model version is set by the user, if true - getting latest version from backend.
    """

    project: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import logging
import re

from collections import defaultdict
from lxml import etree

from label_studio_sdk._extensions.label_studio_tools.core.utils.exceptions import (
    LabelStudioXMLSyntaxErrorSentryIgnored,
)

logger = logging.getLogger(__name__)

_LABEL_TAGS = {"Label", "Choice", "Relation"}
_NOT_CONTROL_TAGS = {
    "Filter",
}
_DIR_APP_NAME = "label-studio"
_VIDEO_TRACKING_TAGS = {"videorectangle"}


def parse_config(config_string):
    """Parse a given Label Studio labeling configuration and return a structured version of the configuration.
    Useful for formatting results for predicted annotations and determining the type(s) of ML models that might
    be relevant to the labeling project.

    :param config_string: Label config string
    :return: structured config of the form:
    {
        "<ControlTag>.name": {
            "type": "ControlTag",
            "to_name": ["<ObjectTag1>.name", "<ObjectTag2>.name"],
            "inputs: [
                {"type": "ObjectTag1", "value": "<ObjectTag1>.value"},
                {"type": "ObjectTag2", "value": "<ObjectTag2>.value"}
            ],
            "labels": ["Label1", "Label2", "Label3"] // taken from "alias" if it exists, else "value"
    }
    """
    if not config_string:
        return {}

    try:
        xml_tree = etree.fromstring(config_string)
    except etree.XMLSyntaxError as e:
        raise LabelStudioXMLSyntaxErrorSentryIgnored(str(e))
    inputs, outputs, labels = {}, {}, defaultdict(dict)
    # Add variables to config (e.g. {{idx}} for index in Repeater
    variables = []
    for tag in xml_tree.iter():
        if tag.attrib and "indexFlag" in tag.attrib:
            variables.append(tag.attrib["indexFlag"])
        if _is_output_tag(tag):
            tag_info = {"type": tag.tag, "to_name": tag.attrib["toName"].split(",")}
            if variables:
                # Find variables in tag_name and regex if find it
                for v in variables:
                    for tag_name in tag_info["to_name"]:
                        if v in tag_name:
                            if "regex" not in tag_info:
                                tag_info["regex"] = {}
                            tag_info["regex"][v] = ".*"
            # Grab conditionals if any
            conditionals = {}
            if tag.attrib.get("perRegion") == "true":
                if tag.attrib.get("whenTagName"):
                    conditionals = {"type": "tag", "name": tag.attrib["whenTagName"]}
                elif tag.attrib.get("whenLabelValue"):
                    conditionals = {
                        "type": "label",
                        "name": tag.attrib["whenLabelValue"],
                    }
                elif tag.attrib.get("whenChoiceValue"):
                    conditionals = {
                        "type": "choice",
                        "name": tag.attrib["whenChoiceValue"],
                    }
            if conditionals:
                tag_info["conditionals"] = conditionals
            if has_variable(tag.attrib.get("value", "")) or tag.attrib.get("apiUrl"):
                tag_info["dynamic_labels"] = True
            outputs[tag.attrib["name"]] = tag_info
        elif _is_input_tag(tag):
            inputs[tag.attrib["name"]] = {
                "type": tag.tag,
                "valueType": tag.attrib.get("valueType"),
            }
            if 'value' in tag.attrib:
                inputs[tag.attrib["name"]]["value"] = tag.attrib["value"].lstrip("$")
            elif 'valueList' in tag.attrib:
                inputs[tag.attrib["name"]]["valueList"] = tag.attrib["valueList"].lstrip("$")
            else:
                raise ValueError(
                    'Inspecting tag {tag_name}... found no "value" or "valueList" attributes.'.format(
                        tag_name=etree.tostring(tag, encoding="unicode").strip()[:50]
                    )
                )
        if tag.tag not in _LABEL_TAGS:
            continue
        parent_name = _get_parent_output_tag_name(tag, outputs)
        if parent_name is not None:
            actual_value = tag.attrib.get("alias") or tag.attrib.get("value") or tag.attrib.get("valueList")
            if not actual_value:
                logger.debug(
                    'Inspecting tag {tag_name}... found no "value", "valueList", or "alias" attributes.'.format(
                        tag_name=etree.tostring(tag, encoding="unicode").strip()[:50]
                    )
                )
            else:
                labels[parent_name][actual_value] = dict(tag.attrib)
    for output_tag, tag_info in outputs.items():
        tag_info["inputs"] = []
        for input_tag_name in tag_info["to_name"]:
            if input_tag_name not in inputs:
                logger.info(
                    f"to_name={input_tag_name} is specified for output tag name={output_tag}, "
                    "but we can't find it among input tags"
                )
                continue
            tag_info["inputs"].append(inputs[input_tag_name])
        tag_info["labels"] = list(labels[output_tag])
        tag_info["labels_attrs"] = labels[output_tag]
    return outputs


def is_video_object_tracking(parsed_config):
    for component in parsed_config:
        if parsed_config[component]["type"].lower() in _VIDEO_TRACKING_TAGS:
            return True


def has_variable(actual_value):
    """
    Check if value has variable
    :param actual_value: value to check
    :return: True if value has variable
    """
    expression = "^\$[A-Za-z_]+$"
    pattern = re.compile(expression)
    full_match = pattern.fullmatch(actual_value)
    if full_match:
        return True
    return False


def _is_input_tag(tag):
    """
    Check if tag is input
    """
    return tag.attrib.get("name") and (tag.attrib.get("value") or tag.attrib.get("valueList"))


def _is_output_tag(tag):
    """
    Check if tag is output
    """
    return (
        tag.attrib.get("name")
        and tag.attrib.get("toName")
        and tag.tag not in _NOT_CONTROL_TAGS
    )


def _get_parent_output_tag_name(tag, outputs):
    # Find parental <Choices> tag for nested tags like <Choices><View><View><Choice>...
    parent = tag
    while True:
        parent = parent.getparent()
        if parent is None:
            return
        name = parent.attrib.get("name")
        if name in outputs:
            return name



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...types.task import Task
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TasksListResponse(UniversalBaseModel):
    tasks: typing.Optional[typing.List[Task]] = pydantic.Field(default=None)
    """
    List of tasks, each task contains predictions and annotations if `fields` query parameter is set to `all`
    """

    total: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of tasks
    """

    total_annotations: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of annotations
    """

    total_predictions: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of predictions
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class PromptsBatchPredictionsRequestResultsItem(UniversalBaseModel):
    task_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Task ID to associate the prediction with
    """

    output: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Prediction output that contains keys from labeling config. Each key must be a valid control tag name from the labeling config. For example, given the output: ```json {"sentiment": "positive"} ``` it will be converted to the internal LS annotation format: ```json {
      "value": {
        "choices": ["positive"]
      },
      "from_name": "label",
      "to_name": "",
      ...
    } ```
    """

    prompt_tokens: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of tokens in the prompt
    """

    completion_tokens: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of tokens in the completion
    """

    prompt_cost_usd: typing.Optional[float] = pydantic.Field(default=None)
    """
    Cost of the prompt (in USD)
    """

    completion_cost_usd: typing.Optional[float] = pydantic.Field(default=None)
    """
    Cost of the completion (in USD)
    """

    total_cost_usd: typing.Optional[float] = pydantic.Field(default=None)
    """
    Total cost of the inference (in USD)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .pause_paused_by import PausePausedBy
import pydantic
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Pause(UniversalBaseModel):
    id: typing.Optional[int] = None
    project: typing.Optional[int] = None
    user: typing.Optional[int] = None
    paused_by: typing.Optional[PausePausedBy] = pydantic.Field(default=None)
    """
    User who created the pause
    """

    reason: str
    verbose_reason: typing.Optional[str] = None
    deleted_by: typing.Optional[int] = None
    deleted_at: typing.Optional[dt.datetime] = None
    created_at: typing.Optional[dt.datetime] = None
    updated_at: typing.Optional[dt.datetime] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .project_import_status import ProjectImportStatus
import datetime as dt
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ProjectImport(UniversalBaseModel):
    id: typing.Optional[int] = None
    preannotated_from_fields: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    commit_to_project: typing.Optional[bool] = None
    return_task_ids: typing.Optional[bool] = None
    status: typing.Optional[ProjectImportStatus] = None
    url: typing.Optional[str] = None
    traceback: typing.Optional[str] = None
    error: typing.Optional[str] = None
    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation time
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Updated time
    """

    finished_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Complete or fail time
    """

    task_count: typing.Optional[int] = None
    annotation_count: typing.Optional[int] = None
    prediction_count: typing.Optional[int] = None
    duration: typing.Optional[int] = None
    file_upload_ids: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    could_be_tasks_list: typing.Optional[bool] = None
    found_formats: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    data_columns: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    tasks: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    task_ids: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    project: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import typing_extensions
from ....core.serialization import FieldMetadata
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class S3UpdateResponse(UniversalBaseModel):
    regex_filter: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cloud storage regex for filtering objects. You must specify it otherwise no objects will be imported.
    """

    use_blob_urls: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Interpret objects as BLOBs and generate URLs. For example, if your bucket contains images, you can use this option to generate URLs for these images. If set to False, it will read the content of the file and load it into Label Studio.
    """

    presign: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Presign URLs for download
    """

    presign_ttl: typing.Optional[int] = pydantic.Field(default=None)
    """
    Presign TTL in minutes
    """

    recursive_scan: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Scan recursively
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Storage description
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID
    """

    bucket: typing.Optional[str] = pydantic.Field(default=None)
    """
    S3 bucket name
    """

    prefix: typing.Optional[str] = pydantic.Field(default=None)
    """
    S3 bucket prefix
    """

    aws_access_key_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS_ACCESS_KEY_ID
    """

    aws_secret_access_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS_SECRET_ACCESS_KEY
    """

    aws_session_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS_SESSION_TOKEN
    """

    aws_sse_kms_key_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS SSE KMS Key ID
    """

    region_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS Region
    """

    s3endpoint: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="s3_endpoint")] = pydantic.Field(
        default=None
    )
    """
    S3 Endpoint
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

InferenceRunStatus = typing.Union[
    typing.Literal["Pending", "InProgress", "Completed", "Failed", "Canceled"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from .members_create_response import MembersCreateResponse
from .members_list_response_item import MembersListResponseItem

__all__ = ["MembersCreateResponse", "MembersListResponseItem"]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    PromptsBatchFailedPredictionsRequestFailedPredictionsItem,
    PromptsBatchFailedPredictionsResponse,
    PromptsBatchPredictionsRequestResultsItem,
    PromptsBatchPredictionsResponse,
)
from . import indicators, runs, versions
from .runs import RunsListRequestProjectSubset

__all__ = [
    "PromptsBatchFailedPredictionsRequestFailedPredictionsItem",
    "PromptsBatchFailedPredictionsResponse",
    "PromptsBatchPredictionsRequestResultsItem",
    "PromptsBatchPredictionsResponse",
    "RunsListRequestProjectSubset",
    "indicators",
    "runs",
    "versions",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.comment import Comment
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.jsonable_encoder import jsonable_encoder
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CommentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        project: typing.Optional[int] = None,
        expand_created_by: typing.Optional[bool] = None,
        annotation: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Comment]:
        """

        Get a list of comments for a specific project.

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        expand_created_by : typing.Optional[bool]
            Expand the created_by field with object instead of ID

        annotation : typing.Optional[int]
            Annotation ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Comment]


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.comments.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/comments/",
            method="GET",
            params={
                "project": project,
                "expand_created_by": expand_created_by,
                "annotation": annotation,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Comment],
                    parse_obj_as(
                        type_=typing.List[Comment],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        annotation: typing.Optional[int] = OMIT,
        project: typing.Optional[int] = OMIT,
        text: typing.Optional[str] = OMIT,
        is_resolved: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Comment:
        """

        Create a new comment.

        Parameters
        ----------
        annotation : typing.Optional[int]

        project : typing.Optional[int]

        text : typing.Optional[str]

        is_resolved : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Comment


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.comments.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/comments/",
            method="POST",
            json={
                "annotation": annotation,
                "project": project,
                "text": text,
                "is_resolved": is_resolved,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Comment,
                    parse_obj_as(
                        type_=Comment,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Comment:
        """

        Get a specific comment.

        Parameters
        ----------
        id : int
            Comment ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Comment


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.comments.get(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/comments/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Comment,
                    parse_obj_as(
                        type_=Comment,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete a specific comment.

        Parameters
        ----------
        id : int
            Comment ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.comments.delete(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/comments/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: int,
        *,
        annotation: typing.Optional[int] = OMIT,
        project: typing.Optional[int] = OMIT,
        text: typing.Optional[str] = OMIT,
        is_resolved: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Comment:
        """

        Update a specific comment.

        Parameters
        ----------
        id : int
            Comment ID

        annotation : typing.Optional[int]

        project : typing.Optional[int]

        text : typing.Optional[str]

        is_resolved : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Comment


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.comments.update(
            id=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/comments/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "annotation": annotation,
                "project": project,
                "text": text,
                "is_resolved": is_resolved,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Comment,
                    parse_obj_as(
                        type_=Comment,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncCommentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        project: typing.Optional[int] = None,
        expand_created_by: typing.Optional[bool] = None,
        annotation: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Comment]:
        """

        Get a list of comments for a specific project.

        Parameters
        ----------
        project : typing.Optional[int]
            Project ID

        expand_created_by : typing.Optional[bool]
            Expand the created_by field with object instead of ID

        annotation : typing.Optional[int]
            Annotation ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Comment]


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.comments.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/comments/",
            method="GET",
            params={
                "project": project,
                "expand_created_by": expand_created_by,
                "annotation": annotation,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Comment],
                    parse_obj_as(
                        type_=typing.List[Comment],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        annotation: typing.Optional[int] = OMIT,
        project: typing.Optional[int] = OMIT,
        text: typing.Optional[str] = OMIT,
        is_resolved: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Comment:
        """

        Create a new comment.

        Parameters
        ----------
        annotation : typing.Optional[int]

        project : typing.Optional[int]

        text : typing.Optional[str]

        is_resolved : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Comment


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.comments.create()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/comments/",
            method="POST",
            json={
                "annotation": annotation,
                "project": project,
                "text": text,
                "is_resolved": is_resolved,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Comment,
                    parse_obj_as(
                        type_=Comment,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> Comment:
        """

        Get a specific comment.

        Parameters
        ----------
        id : int
            Comment ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Comment


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.comments.get(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/comments/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Comment,
                    parse_obj_as(
                        type_=Comment,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """

        Delete a specific comment.

        Parameters
        ----------
        id : int
            Comment ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.comments.delete(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/comments/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: int,
        *,
        annotation: typing.Optional[int] = OMIT,
        project: typing.Optional[int] = OMIT,
        text: typing.Optional[str] = OMIT,
        is_resolved: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Comment:
        """

        Update a specific comment.

        Parameters
        ----------
        id : int
            Comment ID

        annotation : typing.Optional[int]

        project : typing.Optional[int]

        text : typing.Optional[str]

        is_resolved : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Comment


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.comments.update(
                id=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/comments/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "annotation": annotation,
                "project": project,
                "text": text,
                "is_resolved": is_resolved,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Comment,
                    parse_obj_as(
                        type_=Comment,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from .prompt_associated_projects_item_id import PromptAssociatedProjectsItemId

PromptAssociatedProjectsItem = typing.Union[int, PromptAssociatedProjectsItemId]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.model_provider_connection import ModelProviderConnection
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.model_provider_connection_provider import ModelProviderConnectionProvider
from ..types.model_provider_connection_scope import ModelProviderConnectionScope
from ..types.model_provider_connection_organization import ModelProviderConnectionOrganization
from ..types.model_provider_connection_created_by import ModelProviderConnectionCreatedBy
import datetime as dt
from ..types.model_provider_connection_budget_reset_period import ModelProviderConnectionBudgetResetPeriod
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.jsonable_encoder import jsonable_encoder
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ModelProvidersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[ModelProviderConnection]:
        """
        Get all model provider connections created by the user in the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ModelProviderConnection]


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.model_providers.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/model-provider-connections/",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[ModelProviderConnection],
                    parse_obj_as(
                        type_=typing.List[ModelProviderConnection],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        provider: ModelProviderConnectionProvider,
        api_key: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        endpoint: typing.Optional[str] = OMIT,
        scope: typing.Optional[ModelProviderConnectionScope] = OMIT,
        organization: typing.Optional[ModelProviderConnectionOrganization] = OMIT,
        created_by: typing.Optional[ModelProviderConnectionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        is_internal: typing.Optional[bool] = OMIT,
        budget_limit: typing.Optional[float] = OMIT,
        budget_last_reset_date: typing.Optional[dt.datetime] = OMIT,
        budget_reset_period: typing.Optional[ModelProviderConnectionBudgetResetPeriod] = OMIT,
        budget_total_spent: typing.Optional[float] = OMIT,
        budget_alert_threshold: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ModelProviderConnection:
        """
        Create a new model provider connection.

        Parameters
        ----------
        provider : ModelProviderConnectionProvider

        api_key : typing.Optional[str]

        deployment_name : typing.Optional[str]

        endpoint : typing.Optional[str]

        scope : typing.Optional[ModelProviderConnectionScope]

        organization : typing.Optional[ModelProviderConnectionOrganization]

        created_by : typing.Optional[ModelProviderConnectionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        is_internal : typing.Optional[bool]
            Whether the model provider connection is internal, not visible to the user.

        budget_limit : typing.Optional[float]
            Budget limit for the model provider connection (null if unlimited)

        budget_last_reset_date : typing.Optional[dt.datetime]
            Date and time the budget was last reset

        budget_reset_period : typing.Optional[ModelProviderConnectionBudgetResetPeriod]
            Budget reset period for the model provider connection (null if not reset)

        budget_total_spent : typing.Optional[float]
            Tracked total budget spent for the given provider connection within the current budget period

        budget_alert_threshold : typing.Optional[float]
            Budget alert threshold for the given provider connection

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelProviderConnection


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.model_providers.create(
            provider="OpenAI",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/model-provider-connections/",
            method="POST",
            json={
                "provider": provider,
                "api_key": api_key,
                "deployment_name": deployment_name,
                "endpoint": endpoint,
                "scope": scope,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=ModelProviderConnectionOrganization, direction="write"
                ),
                "created_by": convert_and_respect_annotation_metadata(
                    object_=created_by, annotation=ModelProviderConnectionCreatedBy, direction="write"
                ),
                "created_at": created_at,
                "updated_at": updated_at,
                "is_internal": is_internal,
                "budget_limit": budget_limit,
                "budget_last_reset_date": budget_last_reset_date,
                "budget_reset_period": budget_reset_period,
                "budget_total_spent": budget_total_spent,
                "budget_alert_threshold": budget_alert_threshold,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ModelProviderConnection,
                    parse_obj_as(
                        type_=ModelProviderConnection,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, pk: int, *, request_options: typing.Optional[RequestOptions] = None) -> ModelProviderConnection:
        """
        Get a model provider connection by ID.

        Parameters
        ----------
        pk : int
            Model Provider Connection ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelProviderConnection


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.model_providers.get(
            pk=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/model-provider-connections/{jsonable_encoder(pk)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ModelProviderConnection,
                    parse_obj_as(
                        type_=ModelProviderConnection,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, pk: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a model provider connection by ID.

        Parameters
        ----------
        pk : int
            Model Provider Connection ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.model_providers.delete(
            pk=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/model-provider-connections/{jsonable_encoder(pk)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        pk: int,
        *,
        provider: ModelProviderConnectionProvider,
        api_key: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        endpoint: typing.Optional[str] = OMIT,
        scope: typing.Optional[ModelProviderConnectionScope] = OMIT,
        organization: typing.Optional[ModelProviderConnectionOrganization] = OMIT,
        created_by: typing.Optional[ModelProviderConnectionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        is_internal: typing.Optional[bool] = OMIT,
        budget_limit: typing.Optional[float] = OMIT,
        budget_last_reset_date: typing.Optional[dt.datetime] = OMIT,
        budget_reset_period: typing.Optional[ModelProviderConnectionBudgetResetPeriod] = OMIT,
        budget_total_spent: typing.Optional[float] = OMIT,
        budget_alert_threshold: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ModelProviderConnection:
        """
        Update a model provider connection by ID.

        Parameters
        ----------
        pk : int
            Model Provider Connection ID

        provider : ModelProviderConnectionProvider

        api_key : typing.Optional[str]

        deployment_name : typing.Optional[str]

        endpoint : typing.Optional[str]

        scope : typing.Optional[ModelProviderConnectionScope]

        organization : typing.Optional[ModelProviderConnectionOrganization]

        created_by : typing.Optional[ModelProviderConnectionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        is_internal : typing.Optional[bool]
            Whether the model provider connection is internal, not visible to the user.

        budget_limit : typing.Optional[float]
            Budget limit for the model provider connection (null if unlimited)

        budget_last_reset_date : typing.Optional[dt.datetime]
            Date and time the budget was last reset

        budget_reset_period : typing.Optional[ModelProviderConnectionBudgetResetPeriod]
            Budget reset period for the model provider connection (null if not reset)

        budget_total_spent : typing.Optional[float]
            Tracked total budget spent for the given provider connection within the current budget period

        budget_alert_threshold : typing.Optional[float]
            Budget alert threshold for the given provider connection

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelProviderConnection


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.model_providers.update(
            pk=1,
            provider="OpenAI",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/model-provider-connections/{jsonable_encoder(pk)}",
            method="PATCH",
            json={
                "provider": provider,
                "api_key": api_key,
                "deployment_name": deployment_name,
                "endpoint": endpoint,
                "scope": scope,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=ModelProviderConnectionOrganization, direction="write"
                ),
                "created_by": convert_and_respect_annotation_metadata(
                    object_=created_by, annotation=ModelProviderConnectionCreatedBy, direction="write"
                ),
                "created_at": created_at,
                "updated_at": updated_at,
                "is_internal": is_internal,
                "budget_limit": budget_limit,
                "budget_last_reset_date": budget_last_reset_date,
                "budget_reset_period": budget_reset_period,
                "budget_total_spent": budget_total_spent,
                "budget_alert_threshold": budget_alert_threshold,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ModelProviderConnection,
                    parse_obj_as(
                        type_=ModelProviderConnection,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncModelProvidersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ModelProviderConnection]:
        """
        Get all model provider connections created by the user in the current organization.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ModelProviderConnection]


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.model_providers.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/model-provider-connections/",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[ModelProviderConnection],
                    parse_obj_as(
                        type_=typing.List[ModelProviderConnection],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        provider: ModelProviderConnectionProvider,
        api_key: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        endpoint: typing.Optional[str] = OMIT,
        scope: typing.Optional[ModelProviderConnectionScope] = OMIT,
        organization: typing.Optional[ModelProviderConnectionOrganization] = OMIT,
        created_by: typing.Optional[ModelProviderConnectionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        is_internal: typing.Optional[bool] = OMIT,
        budget_limit: typing.Optional[float] = OMIT,
        budget_last_reset_date: typing.Optional[dt.datetime] = OMIT,
        budget_reset_period: typing.Optional[ModelProviderConnectionBudgetResetPeriod] = OMIT,
        budget_total_spent: typing.Optional[float] = OMIT,
        budget_alert_threshold: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ModelProviderConnection:
        """
        Create a new model provider connection.

        Parameters
        ----------
        provider : ModelProviderConnectionProvider

        api_key : typing.Optional[str]

        deployment_name : typing.Optional[str]

        endpoint : typing.Optional[str]

        scope : typing.Optional[ModelProviderConnectionScope]

        organization : typing.Optional[ModelProviderConnectionOrganization]

        created_by : typing.Optional[ModelProviderConnectionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        is_internal : typing.Optional[bool]
            Whether the model provider connection is internal, not visible to the user.

        budget_limit : typing.Optional[float]
            Budget limit for the model provider connection (null if unlimited)

        budget_last_reset_date : typing.Optional[dt.datetime]
            Date and time the budget was last reset

        budget_reset_period : typing.Optional[ModelProviderConnectionBudgetResetPeriod]
            Budget reset period for the model provider connection (null if not reset)

        budget_total_spent : typing.Optional[float]
            Tracked total budget spent for the given provider connection within the current budget period

        budget_alert_threshold : typing.Optional[float]
            Budget alert threshold for the given provider connection

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelProviderConnection


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.model_providers.create(
                provider="OpenAI",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/model-provider-connections/",
            method="POST",
            json={
                "provider": provider,
                "api_key": api_key,
                "deployment_name": deployment_name,
                "endpoint": endpoint,
                "scope": scope,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=ModelProviderConnectionOrganization, direction="write"
                ),
                "created_by": convert_and_respect_annotation_metadata(
                    object_=created_by, annotation=ModelProviderConnectionCreatedBy, direction="write"
                ),
                "created_at": created_at,
                "updated_at": updated_at,
                "is_internal": is_internal,
                "budget_limit": budget_limit,
                "budget_last_reset_date": budget_last_reset_date,
                "budget_reset_period": budget_reset_period,
                "budget_total_spent": budget_total_spent,
                "budget_alert_threshold": budget_alert_threshold,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ModelProviderConnection,
                    parse_obj_as(
                        type_=ModelProviderConnection,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, pk: int, *, request_options: typing.Optional[RequestOptions] = None) -> ModelProviderConnection:
        """
        Get a model provider connection by ID.

        Parameters
        ----------
        pk : int
            Model Provider Connection ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelProviderConnection


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.model_providers.get(
                pk=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/model-provider-connections/{jsonable_encoder(pk)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ModelProviderConnection,
                    parse_obj_as(
                        type_=ModelProviderConnection,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, pk: int, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a model provider connection by ID.

        Parameters
        ----------
        pk : int
            Model Provider Connection ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.model_providers.delete(
                pk=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/model-provider-connections/{jsonable_encoder(pk)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        pk: int,
        *,
        provider: ModelProviderConnectionProvider,
        api_key: typing.Optional[str] = OMIT,
        deployment_name: typing.Optional[str] = OMIT,
        endpoint: typing.Optional[str] = OMIT,
        scope: typing.Optional[ModelProviderConnectionScope] = OMIT,
        organization: typing.Optional[ModelProviderConnectionOrganization] = OMIT,
        created_by: typing.Optional[ModelProviderConnectionCreatedBy] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        is_internal: typing.Optional[bool] = OMIT,
        budget_limit: typing.Optional[float] = OMIT,
        budget_last_reset_date: typing.Optional[dt.datetime] = OMIT,
        budget_reset_period: typing.Optional[ModelProviderConnectionBudgetResetPeriod] = OMIT,
        budget_total_spent: typing.Optional[float] = OMIT,
        budget_alert_threshold: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ModelProviderConnection:
        """
        Update a model provider connection by ID.

        Parameters
        ----------
        pk : int
            Model Provider Connection ID

        provider : ModelProviderConnectionProvider

        api_key : typing.Optional[str]

        deployment_name : typing.Optional[str]

        endpoint : typing.Optional[str]

        scope : typing.Optional[ModelProviderConnectionScope]

        organization : typing.Optional[ModelProviderConnectionOrganization]

        created_by : typing.Optional[ModelProviderConnectionCreatedBy]

        created_at : typing.Optional[dt.datetime]

        updated_at : typing.Optional[dt.datetime]

        is_internal : typing.Optional[bool]
            Whether the model provider connection is internal, not visible to the user.

        budget_limit : typing.Optional[float]
            Budget limit for the model provider connection (null if unlimited)

        budget_last_reset_date : typing.Optional[dt.datetime]
            Date and time the budget was last reset

        budget_reset_period : typing.Optional[ModelProviderConnectionBudgetResetPeriod]
            Budget reset period for the model provider connection (null if not reset)

        budget_total_spent : typing.Optional[float]
            Tracked total budget spent for the given provider connection within the current budget period

        budget_alert_threshold : typing.Optional[float]
            Budget alert threshold for the given provider connection

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModelProviderConnection


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.model_providers.update(
                pk=1,
                provider="OpenAI",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/model-provider-connections/{jsonable_encoder(pk)}",
            method="PATCH",
            json={
                "provider": provider,
                "api_key": api_key,
                "deployment_name": deployment_name,
                "endpoint": endpoint,
                "scope": scope,
                "organization": convert_and_respect_annotation_metadata(
                    object_=organization, annotation=ModelProviderConnectionOrganization, direction="write"
                ),
                "created_by": convert_and_respect_annotation_metadata(
                    object_=created_by, annotation=ModelProviderConnectionCreatedBy, direction="write"
                ),
                "created_at": created_at,
                "updated_at": updated_at,
                "is_internal": is_internal,
                "budget_limit": budget_limit,
                "budget_last_reset_date": budget_last_reset_date,
                "budget_reset_period": budget_reset_period,
                "budget_total_spent": budget_total_spent,
                "budget_alert_threshold": budget_alert_threshold,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ModelProviderConnection,
                    parse_obj_as(
                        type_=ModelProviderConnection,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .user_simple import UserSimple
import datetime as dt
import pydantic
from .export_snapshot_status import ExportSnapshotStatus
from .converted_format import ConvertedFormat
from .task_filter_options import TaskFilterOptions
from .annotation_filter_options import AnnotationFilterOptions
from .serialization_options import SerializationOptions
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ExportSnapshot(UniversalBaseModel):
    title: typing.Optional[str] = None
    id: typing.Optional[int] = None
    created_by: typing.Optional[UserSimple] = None
    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation time
    """

    finished_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Complete or fail time
    """

    status: typing.Optional[ExportSnapshotStatus] = None
    md5: typing.Optional[str] = None
    counters: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None
    converted_formats: typing.Optional[typing.List[ConvertedFormat]] = None
    task_filter_options: typing.Optional[TaskFilterOptions] = None
    annotation_filter_options: typing.Optional[AnnotationFilterOptions] = None
    serialization_options: typing.Optional[SerializationOptions] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .filter_group import FilterGroup
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class View(UniversalBaseModel):
    id: typing.Optional[int] = None
    filter_group: typing.Optional[FilterGroup] = None
    data: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Custom view data
    """

    ordering: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Ordering parameters
    """

    selected_items: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Selected items
    """

    user: typing.Optional[int] = pydantic.Field(default=None)
    """
    User who made this view
    """

    project: int = pydantic.Field()
    """
    Project ID
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
import typing
from ...core.request_options import RequestOptions
from ...types.key_indicators import KeyIndicators
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.key_indicator_value import KeyIndicatorValue
from ...core.client_wrapper import AsyncClientWrapper


class IndicatorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(self, pk: int, *, request_options: typing.Optional[RequestOptions] = None) -> KeyIndicators:
        """
        Get key indicators for the Prompt dashboard.

        Parameters
        ----------
        pk : int
            Inference run ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KeyIndicators


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.indicators.list(
            pk=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/inference-runs/{jsonable_encoder(pk)}/indicators",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    KeyIndicators,
                    parse_obj_as(
                        type_=KeyIndicators,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, indicator_key: str, pk: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> KeyIndicatorValue:
        """
        Get a specific key indicator for the Prompt dashboard.

        Parameters
        ----------
        indicator_key : str
            Key of the indicator

        pk : int
            Inference run ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KeyIndicatorValue


        Examples
        --------
        from label_studio_sdk import LabelStudio

        client = LabelStudio(
            api_key="YOUR_API_KEY",
        )
        client.prompts.indicators.get(
            indicator_key="indicator_key",
            pk=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/inference-runs/{jsonable_encoder(pk)}/indicators/{jsonable_encoder(indicator_key)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    KeyIndicatorValue,
                    parse_obj_as(
                        type_=KeyIndicatorValue,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncIndicatorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(self, pk: int, *, request_options: typing.Optional[RequestOptions] = None) -> KeyIndicators:
        """
        Get key indicators for the Prompt dashboard.

        Parameters
        ----------
        pk : int
            Inference run ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KeyIndicators


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.prompts.indicators.list(
                pk=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/inference-runs/{jsonable_encoder(pk)}/indicators",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    KeyIndicators,
                    parse_obj_as(
                        type_=KeyIndicators,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, indicator_key: str, pk: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> KeyIndicatorValue:
        """
        Get a specific key indicator for the Prompt dashboard.

        Parameters
        ----------
        indicator_key : str
            Key of the indicator

        pk : int
            Inference run ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KeyIndicatorValue


        Examples
        --------
        import asyncio

        from label_studio_sdk import AsyncLabelStudio

        client = AsyncLabelStudio(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.prompts.indicators.get(
                indicator_key="indicator_key",
                pk=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/inference-runs/{jsonable_encoder(pk)}/indicators/{jsonable_encoder(indicator_key)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    KeyIndicatorValue,
                    parse_obj_as(
                        type_=KeyIndicatorValue,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



import json
import time
import asyncio
import typing
import pandas as pd
from .client import ExportsClient, AsyncExportsClient
from io import BytesIO
from label_studio_sdk.versions.client import VersionsClient, AsyncVersionsClient
from label_studio_sdk.core.api_error import ApiError


class ExportTimeoutError(ApiError):
    
    def __init__(self, export_snapshot):
        super().__init__(
            status_code=500,
            body=(
                f"Export job timed out after {timeout} seconds: "
                f"unable to retrieve export job {export_snapshot.id}. "
                f"Current status: {export_snapshot.status}. "
                f"Try manually checking the running job with "
                f"`ls.projects.exports.get(project_id={project_id}, export_pk={export_snapshot.id})`."
            )
        )
        
class ExportFailedError(ApiError):
    
    def __init__(self, export_snapshot):
        super().__init__(
            status_code=500,
            body=f"Export failed: {export_snapshot}"
        )


def _check_status(export_snapshot, converted_format_id, status):
    if converted_format_id:
        converted_format = next((c for c in export_snapshot.converted_formats if c.id == converted_format_id), None)
        if converted_format and converted_format.status == status:
            return True
    else:
        if export_snapshot.status == status:
            return True
    return False


class ExportsClientExt(ExportsClient):
    
    def _bytestream_to_fileobj(self, bytestream: typing.Iterable[bytes] | bytes) -> typing.BinaryIO:
        buffer = BytesIO()
        if isinstance(bytestream, typing.Iterable): 
            for chunk in bytestream:
                buffer.write(chunk)
        else:
            buffer.write(bytestream)
        buffer.seek(0)
        return buffer
    
    def _bytestream_to_binary(self, bytestream: typing.Iterable[bytes]) -> bytes:
        fileobj = self._bytestream_to_fileobj(bytestream)
        return fileobj.getvalue()
    
    def _bytestream_to_json(self, bytestream: typing.Iterable[bytes]) -> dict:
        fileobj = self._bytestream_to_fileobj(bytestream)
        return json.load(fileobj)
    
    def _bytestream_to_pandas(self, bytestream: typing.Iterable[bytes]) -> pd.DataFrame:
        fileobj = self._bytestream_to_fileobj(bytestream)
        return pd.read_csv(fileobj)
    
    def _poll_export(self, project_id, export_snapshot, converted_format_id, timeout):
        start_time = time.time()
        while not _check_status(export_snapshot, None, 'completed'):
            export_snapshot = self.get(project_id, export_pk=export_snapshot.id)
            if _check_status(export_snapshot, None, 'failed'):
                raise ExportFailedError(export_snapshot)
            if time.time() - start_time > timeout:
                raise ExportTimeoutError(export_snapshot)
            time.sleep(1)
    
    def _get_bytestream(
        self,
        project_id: int,
        export_type: str,
        timeout: int = 60,
        create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None,
        convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None,
        download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None,
    ):
        version = VersionsClient(client_wrapper=self._client_wrapper).get()
        
        if version.edition == "Enterprise":
            # Enterprise edition exports are async, so we need to wait for the export job to complete
            export_snapshot = self.create(project_id, **(create_kwargs or {}))
            # Poll for base (JSON) export to complete
            self._poll_export(project_id, export_snapshot, None, timeout)
            # Convert to requested format if not JSON
            if export_type != "JSON":
                converted_proc = self.convert(project_id, export_pk=export_snapshot.id, export_type=export_type, **(convert_kwargs or {}))
                self._poll_export(project_id, export_snapshot, converted_proc.converted_format, timeout)
                
            bytestream = self.download(project_id, export_pk=export_snapshot.id, export_type=export_type, request_options={'chunk_size': 1024}, **(download_kwargs or {}))
        else:
            # Community edition exports are sync, so we can download the file immediately
            bytestream = self.download_sync(project_id, export_type=export_type, download_all_tasks=True, download_resources=True)
        return bytestream
    
    def as_file(self, project_id: int, export_type: str = "JSON", timeout: int = 60, create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None):
        bytestream = self._get_bytestream(project_id, export_type, timeout, create_kwargs, convert_kwargs, download_kwargs)
        return self._bytestream_to_fileobj(bytestream)
    
    def as_binary(self, project_id: int, export_type: str = "JSON", timeout: int = 60, create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None):
        bytestream = self._get_bytestream(project_id, export_type, timeout, create_kwargs, convert_kwargs, download_kwargs)
        return self._bytestream_to_binary(bytestream)
    
    def as_json(self, project_id: int, timeout: int = 60, create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None):
        bytestream = self._get_bytestream(project_id, "JSON", timeout, create_kwargs, convert_kwargs, download_kwargs)
        return self._bytestream_to_json(bytestream)
    
    def as_pandas(self, project_id: int, timeout: int = 60, create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None):
        bytestream = self._get_bytestream(project_id, "CSV", timeout, create_kwargs, convert_kwargs, download_kwargs)
        return self._bytestream_to_pandas(bytestream)
    
class AsyncExportsClientExt(AsyncExportsClient):
    
    async def _bytestream_to_fileobj(self, bytestream: typing.AsyncGenerator[bytes, None] | bytes):
        """Convert bytestream to file-like object"""
        fileobj = BytesIO()
        if isinstance(bytestream, typing.AsyncGenerator):
            async for chunk in bytestream:
                fileobj.write(chunk)
        else:
            fileobj.write(bytestream)
        fileobj.seek(0)
        return fileobj

    async def _bytestream_to_binary(self, bytestream):
        """Convert bytestream to binary data"""
        fileobj = await self._bytestream_to_fileobj(bytestream)
        return fileobj.getvalue()

    async def _bytestream_to_json(self, bytestream):
        """Convert bytestream to JSON object"""
        fileobj = await self._bytestream_to_fileobj(bytestream)
        return json.load(fileobj)

    async def _bytestream_to_pandas(self, bytestream):
        """Convert bytestream to pandas DataFrame"""
        fileobj = await self._bytestream_to_fileobj(bytestream)
        return pd.read_csv(fileobj)
    
    async def _poll_export(self, project_id, export_snapshot, converted_format_id, timeout):
        start_time = time.time()
        while not _check_status(export_snapshot, None, 'completed'):
            export_snapshot = await self.get(project_id, export_pk=export_snapshot.id)
            if _check_status(export_snapshot, None, 'failed'):
                raise ExportFailedError(export_snapshot)
            if time.time() - start_time > timeout:
                raise ExportTimeoutError(export_snapshot)
            await asyncio.sleep(1)

    async def _get_bytestream(
        self,
        project_id: int,
        export_type: str,
        timeout: int = 60,
        create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None,
        convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None,
        download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None,
    ):
        version = await AsyncVersionsClient(client_wrapper=self._client_wrapper).get()
        if version.edition == "Enterprise":
            # Enterprise edition exports are async, so we need to wait for the export job to complete
            export_snapshot = await self.create(project_id, **(create_kwargs or {}))
            # Poll for base (JSON) export to complete
            await self._poll_export(project_id, export_snapshot, None, timeout)
            # Convert to requested format if not JSON
            if export_type != "JSON":
                converted_proc = await self.convert(project_id, export_pk=export_snapshot.id, export_type=export_type, **(convert_kwargs or {}))
                await self._poll_export(project_id, export_snapshot, converted_proc.converted_format, timeout)
            
            bytestream = self.download(project_id, export_pk=export_snapshot.id, export_type=export_type, request_options={'chunk_size': 1024}, **(download_kwargs or {}))
        else:
            bytestream = self.download_sync(project_id, export_type=export_type, download_all_tasks=True, download_resources=True)
        return bytestream
    
    async def as_file(self, project_id: int, export_type: str = "JSON", timeout: int = 60, create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None):
        bytestream = await self._get_bytestream(project_id, export_type, timeout, create_kwargs, convert_kwargs, download_kwargs)
        return await self._bytestream_to_fileobj(bytestream)
    
    async def as_binary(self, project_id: int, export_type: str = "JSON", timeout: int = 60, create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None):
        bytestream = await self._get_bytestream(project_id, export_type, timeout, create_kwargs, convert_kwargs, download_kwargs)
        return await self._bytestream_to_binary(bytestream)
    
    async def as_json(self, project_id: int, timeout: int = 60, create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None):
        bytestream = await self._get_bytestream(project_id, "JSON", timeout, create_kwargs, convert_kwargs, download_kwargs)
        return await self._bytestream_to_json(bytestream)
    
    async def as_pandas(self, project_id: int, timeout: int = 60, create_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, convert_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None, download_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = None):
        bytestream = await self._get_bytestream(project_id, "CSV", timeout, create_kwargs, convert_kwargs, download_kwargs)
        return await self._bytestream_to_pandas(bytestream)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .views_create_request_data_filters_conjunction import ViewsCreateRequestDataFiltersConjunction
import pydantic
import typing
from .views_create_request_data_filters_items_item import ViewsCreateRequestDataFiltersItemsItem
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ViewsCreateRequestDataFilters(UniversalBaseModel):
    """
    Filters to apply on tasks. You can use [the helper class `Filters` from this page](https://labelstud.io/sdk/data_manager.html) to create Data Manager Filters.<br>Example: `{"conjunction": "or", "items": [{"filter": "filter:tasks:completed_at", "operator": "greater", "type": "Datetime", "value": "2021-01-01T00:00:00.000Z"}]}`
    """

    conjunction: ViewsCreateRequestDataFiltersConjunction = pydantic.Field()
    """
    Logical conjunction for the filters. This conjunction (either "or" or "and") will be applied to all items in the filters list. It is not possible to combine "or" and "and" within one list of filters. All filters will be either combined with "or" or with "and", but not a mix of both.
    """

    items: typing.List[ViewsCreateRequestDataFiltersItemsItem] = pydantic.Field()
    """
    List of filter items
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....core.pydantic_utilities import UniversalBaseModel
import typing
from ....types.export_format import ExportFormat
import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2


class ExportsListFormatsResponseItem(UniversalBaseModel):
    name: typing.Optional[ExportFormat] = None
    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Export format title
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Export format description
    """

    link: typing.Optional[str] = pydantic.Field(default=None)
    """
    Export format documentation link
    """

    tags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Export format tags
    """

    disabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, the export format is not supported by the project.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from . import members
from .members import MembersCreateResponse, MembersListResponseItem

__all__ = ["MembersCreateResponse", "MembersListResponseItem", "members"]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .data_manager_task_serializer_predictions_item import DataManagerTaskSerializerPredictionsItem
import pydantic
from .annotations_dm_field import AnnotationsDmField
from .data_manager_task_serializer_drafts_item import DataManagerTaskSerializerDraftsItem
from .data_manager_task_serializer_annotators_item import DataManagerTaskSerializerAnnotatorsItem
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DataManagerTaskSerializer(UniversalBaseModel):
    id: typing.Optional[int] = None
    predictions: typing.Optional[typing.List[DataManagerTaskSerializerPredictionsItem]] = pydantic.Field(default=None)
    """
    Predictions for this task
    """

    annotations: typing.Optional[typing.List[AnnotationsDmField]] = None
    drafts: typing.Optional[typing.List[DataManagerTaskSerializerDraftsItem]] = pydantic.Field(default=None)
    """
    Drafts for this task
    """

    annotators: typing.Optional[typing.List[DataManagerTaskSerializerAnnotatorsItem]] = pydantic.Field(default=None)
    """
    Annotators who annotated this task
    """

    inner_id: typing.Optional[int] = None
    cancelled_annotations: typing.Optional[int] = None
    total_annotations: typing.Optional[int] = None
    total_predictions: typing.Optional[int] = None
    completed_at: typing.Optional[dt.datetime] = None
    annotations_results: typing.Optional[str] = None
    predictions_results: typing.Optional[str] = None
    predictions_score: typing.Optional[float] = None
    file_upload: typing.Optional[str] = None
    storage_filename: typing.Optional[str] = None
    annotations_ids: typing.Optional[str] = None
    predictions_model_versions: typing.Optional[str] = None
    avg_lead_time: typing.Optional[float] = None
    draft_exists: typing.Optional[bool] = None
    updated_by: typing.Optional[typing.List[typing.Dict[str, typing.Optional[typing.Any]]]] = pydantic.Field(
        default=None
    )
    """
    User IDs who updated this task
    """

    data: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    User imported or uploaded data for a task. Data is formatted according to the project label config. You can find examples of data for your project on the Import page in the Label Studio Data Manager UI.
    """

    meta: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Meta is user imported (uploaded) data and can be useful as input for an ML Backend for embeddings, advanced vectors, and other info. It is passed to ML during training/predicting steps.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Time a task was created
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last time a task was updated
    """

    is_labeled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the number of annotations for this task is greater than or equal to the number of maximum_completions for the project
    """

    overlap: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of distinct annotators that processed the current task
    """

    comment_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of comments in the task including all annotations
    """

    unresolved_comment_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of unresolved comments in the task including all annotations
    """

    last_comment_updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When the last comment was updated
    """

    project: typing.Optional[int] = pydantic.Field(default=None)
    """
    Project ID for this task
    """

    comment_authors: typing.Optional[typing.List[int]] = pydantic.Field(default=None)
    """
    Users who wrote comments
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .exports_convert_response import ExportsConvertResponse
from .exports_list_formats_response_item import ExportsListFormatsResponseItem

__all__ = ["ExportsConvertResponse", "ExportsListFormatsResponseItem"]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
import datetime as dt
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Prediction(UniversalBaseModel):
    id: typing.Optional[int] = None
    result: typing.List[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field()
    """
    List of prediction results for the task
    """

    model_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    Model version - tag for predictions that can be used to filter tasks in Data Manager, as well as select specific model version for showing preannotations in the labeling interface
    """

    created_ago: typing.Optional[str] = pydantic.Field(default=None)
    """
    Delta time from creation time
    """

    score: typing.Optional[float] = pydantic.Field(default=None)
    """
    Prediction score
    """

    cluster: typing.Optional[int] = pydantic.Field(default=None)
    """
    Cluster for the current prediction
    """

    neighbors: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Array of task IDs of the closest neighbors
    """

    mislabeling: typing.Optional[float] = pydantic.Field(default=None)
    """
    Related task mislabeling score
    """

    created_at: typing.Optional[dt.datetime] = None
    updated_at: typing.Optional[dt.datetime] = None
    model: typing.Optional[int] = pydantic.Field(default=None)
    """
    An ML Backend instance that created the prediction.
    """

    model_run: typing.Optional[int] = pydantic.Field(default=None)
    """
    A run of a ModelVersion that created the prediction.
    """

    task: int
    project: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
